<?php
/*
** +---------------------------------------------------+
** |
** | Name :		~/main/class/class_upload.php
** | Begin :	29/06/2005
** | Last :		30/09/2007
** | User :		Genova
** | Project :	Fire-Soft-Board 2 - Copyright FSB group
** | License :	GPL v2.0
** +---------------------------------------------------+
*/

/*
** Permet l'upload de fichiers
*/
class Upload extends Fsb_model
{
	// Nom du champ de formulaire d'upload
	protected $name;

	// Informations sur le fichier uploadé
	protected $tmp_path;
	public $filesize;
	public $filename;
	public $basename;
	public $extension;
	public $mimetype;

	// Renommer le fichier
	public $rename_basename = NULL;
	public $rename_extension = NULL;

	// Informations sur l'image
	public $width;
	public $height;
	public $type;

	// Types d'images autorisées
	protected $only_img = FALSE;
	public $is_img = FALSE;
	public static $img = array(
		'gif',
		'jpg',
		'jpeg',
		'bmp',
		'png',
	);
	public static $imgtype = array(
		IMAGETYPE_GIF,
		IMAGETYPE_JPEG,
		IMAGETYPE_JPEG,
		IMAGETYPE_BMP,
		IMAGETYPE_PNG,
	);

	// Extensions autorisées
	protected $allowed_ext = array();

	/*
	** Constructeur
	** -----
	** $name ::		Nom du champ du formulaire d'upload
	*/
	public function __construct($name)
	{
		$this->name = $name;

		if (!isset($_FILES[$this->name]))
		{
			trigger_error("La variable \$_FILES['$this->name'] n'existe pas", FSB_ERROR);
		}

		$this->tmp_path =	$_FILES[$this->name]['tmp_name'];
		$this->filename =	$_FILES[$this->name]['name'];
		$this->mimetype =	$_FILES[$this->name]['type'];
		$this->filesize =	filesize($this->tmp_path);
		$this->basename =	get_file_data($this->filename, 'filename');
		$this->extension =	get_file_data($this->filename, 'extension');

		$ini_size = ini_get_bytes(ini_get('upload_max_filesize'));
		if ($this->filesize > $ini_size)
		{
			Display::message(sprintf(Fsb::$session->lang('bad_ini_size_upload'), $ini_size, $this->filesize));
		}
	}

	/*
	** Ajoute une ou plusieurs extensions autorisées
	** -----
	** $ext ::		Extension ou tableau d'extensions. TRUE pour tout autoriser
	*/
	public function allow_ext($ext)
	{
		if ($ext === TRUE)
		{
			$this->allowed_ext = TRUE;
		}
		else if (is_array($ext))
		{
			$this->allowed_ext = array_merge($this->allowed_ext, $ext);
		}
		else
		{
			$this->allowed_ext[] = $ext;
		}
	}

	/*
	** N'autorise que les images
	** -----
	** $bool ::		TRUE pour n'autoriser que les images
	*/
	public function only_img($bool = TRUE)
	{
		$this->allow_ext(self::$img);
		$this->only_img = $bool;

		// Récupération des informations sur l'image, et vérification de l'intégrité de l'image avec un getimagesize()
		list($this->width, $this->height, $this->type) = @getimagesize($this->tmp_path);
		if (!in_array($this->type, self::$imgtype))
		{
			Display::message('L\'image ' . $this->filename . ' contient des erreurs et ne peut pas être uploadée sur le forum');
		}
	}

	/*
	** Vérification des dimensions de l'image
	** -----
	** $width ::	Largeur max de l'image
	** $height ::	Hauteur max de l'image
	** $filesize ::	Taille max de l'image (en octet)
	*/
	public function check_img_size($width, $height, $filesize)
	{
		if (($width > 0 && $this->width > $width) || ($height > 0 && $this->height > $height) || ($filesize > 0 && $this->filesize > $filesize))
		{
			return (FALSE);
		}
		return (TRUE);
	}

	/*
	** Renomme la partie avant l'extension du fichier
	** -----
	** $new ::		Nouveau nom
	*/
	public function rename_filename($new)
	{
		$this->rename_basename = $new;
	}

	/*
	** Renomme l'extension
	** -----
	** $new ::		Nouveau nom
	*/
	public function rename_extension($new)
	{
		$this->rename_extension = $new;
	}

	/*
	** Upload et sauve l'image sur le forum
	** -----
	** $path ::		Dossier de destination
	** $erase ::	Si TRUE, on écrase le fichier de destination s'il a le même nom
	**				Si FALSE, on renomme le fichier
	*/
	public function store($path, $erase = FALSE)
	{
		// Vérification de l'extension
		if ($this->allowed_ext !== TRUE && !in_array(strtolower($this->extension), $this->allowed_ext))
		{
			Display::message(sprintf(Fsb::$session->lang('bad_extension'), $this->extension, implode(', ', $this->allowed_ext)));
		}

		// On vérifie si le fichier n'existe pas déjà, si c'est le cas on le renomme
		$extension =	($this->rename_extension) ? $this->rename_extension : $this->extension;
		$basename =		($this->rename_basename) ? $this->rename_basename : $this->basename;
		if (!$erase)
		{
			$i = 1;
			$base = $basename;
			while (file_exists($path . $base . '.' . $extension))
			{
				$base = $basename . $i;
				$i++;
			}
			$this->filename = $base . '.' . $extension;
		}

		// Upload du fichier
		if (!(is_uploaded_file($this->tmp_path) && is_writable($path) && move_uploaded_file($this->tmp_path, $path . $this->filename)))
		{
			Display::message(sprintf(Fsb::$session->lang('unable_upload_file'), $this->tmp_path, $path . $this->filename));
		}
		@chmod($path . $this->filename, 0644);

		// S'il s'agit d'une image, on vérifie qu'elle ne contient pas de PHP
		if ($this->only_img)
		{
			if (strpos(file_get_contents($path . $this->filename), '<?php'))
			{
				Display::message('Tentative de hack, l\'image ne peut être uploadée sur le forum');
			}
		}

		return ($this->filename);
	}

	/*
	** Upload de fichier joint pour le forum
	** -----
	** $user_id ::		ID du membre
	** $upload_auth ::	Droit sur le fichier
	*/
	public function attach_file($user_id, $upload_auth)
	{
		// On récupère la taille prise par tous les fichiers uploadé du membre, afin de déterminé le quota restant
		$sql = 'SELECT SUM(upload_filesize) AS total_filesize
				FROM ' . SQL_PREFIX . 'upload
				WHERE u_id = ' . $user_id;
		$result = Fsb::$db->query($sql);
		$row = Fsb::$db->row($result);
		Fsb::$db->free($result);
		$total_filesize = $row['total_filesize'];

		// Quota dépassé ?
		if (!Fsb::$session->is_authorized('upload_quota_unlimited') && $total_filesize > Fsb::$cfg->get('upload_quota'))
		{
			Display::message(sprintf(Fsb::$session->lang('post_upload_quota'), $total_filesize, Fsb::$cfg->get('upload_quota')));
		}

		// Liste des extensions autorisées
		$this->allow_ext(explode(',', Fsb::$cfg->get('upload_extensions')));

		// Taille maximale possible pour le fichier
		$max_size = min(Fsb::$cfg->get('upload_quota') - $total_filesize, Fsb::$cfg->get('upload_max_filesize'));
		if (!$this->check_img_size(0, 0,  $max_size))
		{
			Display::message(sprintf(Fsb::$session->lang('size_too_big'), $this->filesize, $max_size));
		}

		// Upload du fichier sur le serveur
		$this->rename_filename(md5($this->filename));
		$this->rename_extension('file');
		$filename = $this->store(ROOT . 'upload/');

		// On vérifie si le fichier est une image
		$img_data = @getimagesize(ROOT . 'upload/' . $filename);
		if ($img_data && in_array($img_data[2], self::$imgtype))
		{
			$this->is_img = TRUE;
			$this->mimetype = image_type_to_mime_type($img_data[2]);
		}
		else
		{
			$this->mimetype = 'application/octetstream';
		}

		// Ajout du fichier uploadé dans la base de donnée
		Fsb::$db->insert('upload', array(
			'u_id' =>				$user_id,
			'upload_filename' =>	$filename,
			'upload_realname' =>	$_FILES[$this->name]['name'],
			'upload_filesize' =>	filesize(ROOT . 'upload/' . $filename),
			'upload_mimetype' =>	$this->mimetype,
			'upload_time' =>		CURRENT_TIME,
			'upload_auth' =>		$upload_auth,
		));

		return (Fsb::$db->last_id());
	}
}

/* EOF */